@@ -310,8 +310,6 @@ namespace shevek
 		{
 		default: // One word
 			s >> ret;
-			if (ret.size () == 0)
-				return false;
 			break;
 		case 'l': // One line
 			std::getline (s, line);
@@ -341,6 +339,8 @@ namespace shevek
 			pos.top () += ret.size ();
 			return true;
 		}
+		if (!s)
+			return false;
 		std::streampos p = s.tellg ();
 		if (p >= 0)
 			pos.top () += p;
@@ -799,8 +799,6 @@ namespace shevek
 		{
 		default: // One word
 			s >> ret;
-			if (ret.size () == 0)
-				return false;
 			break;
 		case 'l': // One line
 			std::getline (s, ret);
@@ -829,6 +827,8 @@ namespace shevek
 			pos.top () += ret.size ();
 			return true;
 		}
+		if (!s)
+			return false;
 		std::streampos p = s.tellg ();
 		if (p >= 0)
 			pos.top () += p;
